//
// Taken from http://www.exfer.net/blog/2008/01/26/a-cross-browser-changecss-function-to-alter-add-or-delete-css-rules/
// Adapted for my needs and improved to handle imported stylesheets -- Martin Backschat.
//
// Example usage:
// cssRuleChanger.replace("#tableTop", "background", /Images(\.[a-zA-Z]+)?/, "Images.blue");


var cssRuleChanger = window.cssRuleChanger || {};

// if regexp == null, then value is used
// if regexp == null and value == null, then the rule is deleted.
cssRuleChanger.replace = function (selector, stylename, regexp, value)
{
  var ss;
  var changed = false; // means nothing has been changed
 
  // make sure there is a stylesheet available
  if (document.styleSheets.length <= 0) {
    if (document.createStyleSheet) {
      document.createStyleSheet();
    } else {
      var styleTag = document.createElement("style");
      styleTag.type = "text/css";
      document.getElementsByTagName("head")[0].appendChild(styleTag);
    }
  }
 
  // convert the stylename to camel case if necessary
  var ccstylename = (stylename || "").replace(/\-(\w)/g, function (m, c) { return (c.toUpperCase()); });
  ccstylename = (ccstylename == 'float') ? 'cssFloat' : ccstylename;
  selector = selector.toLowerCase();

  // loop to delete or change the css
  for (var i = 0; i < document.styleSheets.length; i++) {
    ss = document.styleSheets[i];
	changed |= cssRuleChanger.replaceInSS(ss, selector, ccstylename, regexp, value);
  }

  return changed;
}



cssRuleChanger.replaceInSS = function (ss, selector, ccstylename, regexp, value)
{
	var rules;
	var changed = false; // means nothing has been changed

    try {
      rules = ss.cssRules || ss.rules;
    } catch (e) {
      rules = [];
    }

    for (var j = 0, len = rules.length; j < len; j++)
	{
		if (rules[j].styleSheet) { // tested only in Safari/Widgets
			changed |= cssRuleChanger.replaceInSS(rules[j].styleSheet, selector, ccstylename, regexp, value);
			continue;
		}

      if (rules[j].selectorText && rules[j].selectorText.toLowerCase() == selector) {
        if (regexp != null || (regexp == null && value != null)) {
          if (regexp == null) {
		    // TODO: this does not work for 10.4 dashboard variant!
            rules[j].style[ccstylename] = value;
          }
          else {
			var s = rules[j].style[ccstylename];
			if (s === undefined || s == null || s.length == 0) {
				// case for 10.4 dashboard variant
            	rules[j].style.cssText = rules[j].style.cssText.replace(regexp, value);
			}
            else {
				// case for 10.5+ dashboard variant
				rules[j].style[ccstylename] = s.replace(regexp, value);
			}
          }
		  changed = true;
        } else {
          if (ss.deleteRule) {
            ss.deleteRule(j);
          } else if (ss.removeRule) {
            ss.removeRule(j);
          } else {
            // poor man's delete
            rules[j].style.cssText = "";
          }
        }
        //return;
      }
	}
	return changed;

  // if (stylename && (regexp != null) && value) {
  //   // if the selector wasn't found and isn't supposed to be deleted then add it
  //   ss = document.styleSheets[0];
  //   rules = ss.cssRules || ss.rules;
  //   if (ss.insertRule) {
  //     ss.insertRule(selector + "{" + stylename + ":" + value + "; }", rules.length);
  //   } else if (ss.addRule) {
  //     ss.addRule(selector, stylename + ":" + value + ";", rules.length);
  //   } else {
  //     throw new Error("Selector not found and add/insert rule not supported.");
  //   }
  // }

}