var XinMonths = new Array;
XinMonths['1m'] = 1;
XinMonths['3m'] = 3;
XinMonths['6m'] = 6;
XinMonths['1y'] = 12;
XinMonths['2y'] = 24;

var N_A_CONST = "N/A";

// Taken from http://www.xml-blog.com/files/date_extras.js
// 30 days = 1 month, so be careful!
var kDayInMs = 24*60*60*1000;
var kMonthInMs = 30*kDayInMs; //  (approx = 1 month)

// Non-destructive instance methods
Date.prototype.addMilliseconds = function(ms){  
  return new Date(new Date().setTime(this.getTime() + (ms)));  
}
Date.prototype.addMonths = function(mm){
  return this.addMilliseconds(mm * kMonthInMs);  
}

Date.prototype.daysDifference = function(d) {
	return Math.abs(Math.ceil((this.getTime()-d.getTime())/kDayInMs));
}

function isCurrencySymbol (symbol) {
	return symbol.indexOf('=') >= 0;
}


var yahooToGoogleSymbolMap = {'^DJA':'DJC'};
var googleToYahooSymbolMap = {'DJC':'^DJA'};

// Additional to the ^DJ* 
var kAdditionalSymbolsNotAvailableInYahoo = {'^W1DOW':1, '^BKTDM':1, '^BKTEM':1 };

function fetchStockData (callback, symbols)
{
	// 1. Filter symbols for Yahoo and Google
	var googleSymbols = "";
	var googleSymbolIndices = [];
	var yahooSymbols = "";
	var yahooSymbolIndices = [];

	var i, s;
	var url;
	var xml_request, xml_request2;
	// Why xml_request2??
	// They are used in closures , cannot be reused!
	// And variables are visible for the whole function scope. Therefore, you should only declare them once.

	for (i = 0; i < symbols.length; i++) {
			s = symbols[i];
			if (s.substr(0,3) == '^DJ'
				|| s.substr(0,3) == '^DW'
                || kAdditionalSymbolsNotAvailableInYahoo.hasOwnProperty(s)) {

                if (yahooToGoogleSymbolMap.hasOwnProperty(s)) {
                    s = yahooToGoogleSymbolMap[s];
                }

                s = s.replace('^', '');
				googleSymbols += 'INDEXDJX:' + s + ',';
                // Remember: google_quotes_json_loaded() must reconstruct the original symbol!
                googleSymbolIndices.push(i);
			}
			else {
				yahooSymbols += s + '+';
                yahooSymbolIndices.push(i);
			}
	}


	// 2. Call Yahoo (if not empty)
	if (yahooSymbols.length > 0) {
		xml_request = new XMLHttpRequest();
		xml_request.onload = function(e) {
			yahoo_quotes_xml_loaded(e, xml_request, callback, yahooSymbolIndices);}
		xml_request.overrideMimeType("text/csv");

		url = 'http://download.finance.yahoo.com/d/quotes.csv?s=' + yahooSymbols + '&f=sl1c&e=.csv';
		xml_request.open("GET", url);
		xml_request.setRequestHeader("Cache-Control", "no-cache");
		xml_request.send(null);
	}

	// 3. Call Google (if not empty)
	if (googleSymbols.length > 0) {
		xml_request2 = new XMLHttpRequest();
		xml_request2.onload = function(e) {
			google_quotes_json_loaded(e, xml_request2, callback, googleSymbolIndices);}

		url = 'http://finance.google.com/finance/info?q=' + googleSymbols;
		xml_request2.open("GET", url);
		xml_request2.setRequestHeader("Cache-Control", "no-cache");
		xml_request2.send(null);
	}
}

function findChild (element, nodeName)
{
	var child;
	
	for (child = element.firstChild; child != null; child = child.nextSibling)
	{
		if (child.nodeName == nodeName) {
			return child;
		}
	}
	
	return null;
}

function findChildWithAttrValue (element, nodeName, attrName, attrValue)
{
	var child;
	
	for (child = element.firstChild; child != null; child = child.nextSibling)
	{
		if (child.nodeName == nodeName && child.getAttribute(attrName) == attrValue) {
			return child;
		}
	}
	
	return null;
}


function constructError (string)
{
	return {error:true, errorString:string};
}



// returns an anonymous object like so
// object
//		error: 	Boolean false for success
//		errorString: failure string
//		quotes: array[n] of anonymous objects like so
//			object
//				symbol:		stock symbol
//				lasttrade:	last trade for symbol
//				change:		delta change for symbol
//				pctchange:	percent change for symbol
//				open:		true if market is open, closed if not

function google_quotes_json_loaded (event, request, callback, symbolIndices)
{
	if (request.responseText) {
		try {
			var obj = {error:false, errorString:null, quotes:[], open:true};

			// extract result array
			var m = request.responseText.slice(3);
			if (m.length < 4) {callback(constructError("No symbol information")); return;}

			var symbolArray = m.parseJSON();	// Secure JSON
			if (symbolArray == null) {callback(constructError("no symbol found")); return;}


			// Example:
			// {
			// "id": "983582"
			// ,"t" : ".DJI"
			// ,"e" : "INDEXDJX"
			// ,"l" : "12,184.26"
			// ,"l_cur" : "12,184.26"
			// ,"s": "0"
			// ,"ltt":"4:04PM EST"
			// ,"lt" : "Dec 9, 4:04PM EST"
			// ,"c" : "+186.56"
			// ,"cp" : "1.55"
			// ,"ccol" : "chg"
			// }

			for (var i = 0; i < symbolArray.length; i++) {
				m = symbolArray[i];
				if (! ('t' in m)) {callback(constructError("invalid symbol information")); return;}

				var sym = m.t;
				if (sym.charAt(0) == '.') { sym = '^' + sym.slice(1); }	// ".DJI" => "^DJI"
                else if (m.e == "INDEXDJX") {
                    if (googleToYahooSymbolMap.hasOwnProperty(sym)) {
                        sym = googleToYahooSymbolMap[sym];
                    }
                    else {
                        sym = '^' + sym;
                    }
                }

				var last = m.l.replace(',', '');	// "12,184.26" => "12184.26"
				var chng = m.c.replace(',', '').replace('+','');
				var pct = m.cp.replace(',', '');
				var MKT_STATUS = true;  // TODO!
				
				if (!MKT_STATUS) { obj.open = false; }

				obj.quotes[obj.quotes.length] = {
					symbol:sym, lasttrade:last,
					change:chng, pctchange:pct, open:MKT_STATUS
				};
			}

			try {
				callback (obj, symbolIndices);
			} catch (ex) {}
		}
		catch (ex)
		{
			callback ({error:true, errorString:"exception thrown when parsing stock symbol lookup data: " + ex});
		}
	}
	else
	{
		callback ({error:true, errorString:"XML request failed. no responseText"});
	}
}


// Test it with this service:
// http://www.regular-expressions.info/javascriptexample.html
//
// Example:
// "SIE.DE",66.56,"+0.38 - +0.57%"
// "EUR=X",0.6869,"N/A - N/A"
var reQuotes = /^"([^"]+)",([\d\.]+),"[+ ]?([^\s]+)\s-\s[+ ]?([^\s\%]+)\%?"/;

function yahoo_quotes_xml_loaded (event, request, callback, symbolIndices)
{
	if (request.responseText) {
		try {
			var obj = {error:false, errorString:null, quotes:[], open:true};

			var quoteLines = request.responseText.split("\n",100);

			if (quoteLines == null || quoteLines.length < 1) {
				callback(constructError("no stock quotes")); return;
			}

			for (var quoteIdx = 0; quoteIdx < quoteLines.length-1; quoteIdx++)
			{
				var m = reQuotes.exec(quoteLines[quoteIdx]);

				if (m.length != 4+1) {callback(constructError("invalid symbol information")); return;}

				var sym = m[1];
				var last = m[2];
				var chng = m[3]; // is a number or N_A_CONST ("N/A")
				var pct = m[4]; // is a number or N_A_CONST ("N/A")
				var MKT_STATUS = true;  // TODO!
				
				if (!MKT_STATUS) { obj.open = false; }

				obj.quotes[obj.quotes.length] = {
					symbol:sym, lasttrade:last,
					change:chng, pctchange:pct, open:MKT_STATUS
				};
			}
		
			try {
				callback (obj, symbolIndices);
			} catch (ex) {}
			
		}
		catch (ex)
		{
			callback ({error:true, errorString:"exception thrown when parsing stock data: " + ex});
		}
	}
	else
	{
		callback ({error:true, errorString:"XML request failed. no responseXML"});
	}
}

// returns an anonymous object like so
// object
//		error: 	Boolean false for success
//		errorString: failure string
//		symbols: array[n] of anonymous objects like so
//			object
//				symbol:		stock symbol
//				name:		company name for symbol
// 				exchange:	the exchange for symbol

function validateSymbol (callback, symbol)
{
	if (isCurrencySymbol(symbol)) {
		return validateSymbol_currency(callback, symbol);
	}

	var url = 'http://autoc.finance.yahoo.com/autoc?query=' + symbol + '&callback=YAHOO.Finance.SymbolSuggest.ssCallback';
	var xml_request = new XMLHttpRequest();
	xml_request.onload = function(e) {validate_symbol_loaded(e, xml_request, callback);}
	//xml_request.overrideMimeType("text/csv");
	xml_request.open("GET", url);
	xml_request.setRequestHeader("Cache-Control", "no-cache");
	xml_request.send(null);
	
	return xml_request;
}

function validate_symbol_loaded (event, request, callback)
{
	if (request.responseText) {
		try {
			var obj = {error:false, errorString:null, symbols:new Array, open:true};

			// extract result array
			var re = /,"Result":(.*)\}\}\)/;
			var m = re.exec(request.responseText);
			if (m.length != 2) {callback(constructError("invalid symbol information")); return;}

			var symbolArray = m[1].parseJSON();	// Secure JSON
			if (symbolArray == null) {callback(constructError("no symbol found")); return;}

			// Adjust symbol object property names:
			// symbol, name : ok
			// exchange : taken from yahoo's exchDisp property
			for (var i = 0; i < symbolArray.length; i++) {
				symbolArray[i].exchange = symbolArray[i].exchDisp;
			}

			obj.symbols = symbolArray;
			
			try {
				callback (obj);
			} catch (ex) {}
		}
		catch (ex)
		{
			callback ({error:true, errorString:"exception thrown when parsing stock symbol lookup data: " + ex});
		}
	}
	else
	{
		callback ({error:true, errorString:"XML request failed. no responseText"});
	}
}


function validateSymbol_currency (callback, symbol)
{
	var url = 'http://finance.yahoo.com/services/1.0/instrument/' + symbol + '/quote?callback=XXXXX';
	var xml_request = new XMLHttpRequest();
	xml_request.onload = function(e) {validate_symbol_loaded_currency(e, xml_request, callback);}
	//xml_request.overrideMimeType("text/csv");
	xml_request.open("GET", url);
	xml_request.setRequestHeader("Cache-Control", "no-cache");
	xml_request.send(null);
	
	return xml_request;
}

function validate_symbol_loaded_currency (event, request, callback)
{
	if (request.responseText) {
		try {
			var obj = {error:false, errorString:null, symbols:new Array, open:true};

			var re = /^([^(]+)\((\{[^}]+\})/;
			var m = re.exec(request.responseText);
			if (m.length != 3) {callback(constructError("invalid symbol information")); return;}

			var StockObj = m[2].parseJSON();	// Secure JSON
			if (StockObj == null || StockObj.valid != "yes") {callback(constructError("no stock found")); return;}

			obj.symbols[obj.symbols.length] = {symbol:StockObj.symbol, name:StockObj.shortname, exchange:''};
			
			try {
				callback (obj);
			} catch (ex) {}
		}
		catch (ex)
		{
			callback ({error:true, errorString:"exception thrown when parsing stock symbol lookup data: " + ex});
		}
	}
	else
	{
		callback ({error:true, errorString:"XML request failed. no responseText"});
	}
}


// returns an anonymous object like so
// object
//		error: 	Boolean false for success
//		errorString: failure string
//		data: array[n] of anonymous objects like so
//			object
//				date:		Date js object
//				close:		close price of stock
function fetchChartData (symbol, code, callback)
{
	// Yahoo Finance Chart API Images
	if (code == '1d' || code == '1w') {
		// $('chart-image').src = "http://uk.ichart.yahoo.com/" + (code == '1d' ? "t" : "v") + "?s=" + symbol + '&m=' + Math.floor(Math.random()*500000);
		// enableChart(true, false);
		// return null;
		fetchChartData_XML_1D_1W(symbol, code, callback);
	}
	else
	if (chartDataSourceState.countdown(symbol, code)) {
		fetchChartData_CSV(symbol, code, callback);
	}
	else {
		fetchChartData_XML(symbol, code, callback);
	}
}

//
// Yahoo Finance API CSV Data
//
function fetchChartData_CSV(symbol, code, callback)
{
	var ysDate = (new Date()).addMonths(- XinMonths[code] - 1); // -1 means: take one additional month
	var url = 'http://ichart.finance.yahoo.com/table.csv?s=' + symbol + '&a=' + ysDate.getMonth() + '&b=' + ysDate.getDate() + '&c=' + ysDate.getFullYear();

	var xml_request = new XMLHttpRequest();
	xml_request.onload = function(e) {chartdata_csvold_loaded(e, xml_request, callback, symbol, code);}
	xml_request.overrideMimeType("text/csv");
	xml_request.open("GET", url);
	xml_request.setRequestHeader("Cache-Control", "no-cache");
	xml_request.send(null);
	return xml_request;
}


//
// New Yahoo Finance Chart API XML Data
//
function fetchChartData_XML(symbol, code, callback)
{
	// var ysDate = (new Date()).addMonths(- XinMonths[code] - 1); // -1 means: take one additional month
	// var ys = ysDate.getFullYear();
	// var yz = (new Date()).getFullYear() + 1 - ys;
	// var url = 'http://chartapi.finance.yahoo.com/instrument/1.0/' + symbol + '/chartdata;type=close;ys=' + ys + ';yz=' + yz + '/xml';
	
	var url = 'http://chartapi.finance.yahoo.com/instrument/1.0/' + symbol + '/chartdata;type=close;range=' + code + '/xml';
	
	var xml_request = new XMLHttpRequest();
	xml_request.onload = function(e) {chartdata_xml_loaded(e, xml_request, callback, symbol, code);}
	xml_request.overrideMimeType("text/xml");
	xml_request.open("GET", url);
	xml_request.setRequestHeader("Cache-Control", "no-cache");
	xml_request.send(null);
	return xml_request;
}

//
// New Yahoo Finance Chart API XML Data for 1day, 5day
//
function fetchChartData_XML_1D_1W(symbol, code, callback)
{
	if (code == '1w') {
		code = '5d';
	}
	var url = 'http://chartapi.finance.yahoo.com/instrument/1.0/' + symbol + '/chartdata;type=close;range=' + code + '/xml';
	
	var xml_request = new XMLHttpRequest();
	xml_request.onload = function(e) {chartdata_xml_1d_1w_loaded(e, xml_request, callback, symbol, code);}
	xml_request.overrideMimeType("text/xml");
	xml_request.open("GET", url);
	xml_request.setRequestHeader("Cache-Control", "no-cache");
	xml_request.send(null);
	return xml_request;
}


// Test it with this service:
// http://www.regular-expressions.info/javascriptexample.html
//
// Example:
// Date,Open,High,Low,Close,Volume,Adj Close
// 2008-03-18,6237.63,6417.95,6237.63,6393.39,000,6393.39

var reCvsOld = /^([\d]+)-([\d]+)-([\d]+),[\d\.]+,[\d\.]+,[\d\.]+,[\d\.]+,[\d\.]+,([\d\.]+)/;

function chartdata_csvold_loaded (event, request, callback, symbol, code)
{
	if (request.responseText) {
		try {
			var obj = {error:false, errorString:null, data:new Array};
			var firstDate = new Date();
			firstDate = firstDate.addMonths(- XinMonths[code]);

			var quoteLines = request.responseText.split("\n");

			if (quoteLines == null || quoteLines.length < 1) {
				callback(constructError("no stock quotes")); return;
			}

			var array = obj.data;
			for (var quoteIdx = quoteLines.length-1; quoteIdx >= 1 /* skip header line */; quoteIdx--)
			{
				if (quoteLines[quoteIdx].length == 0) {continue}
				var m = reCvsOld.exec(quoteLines[quoteIdx]);

				if (m.length != 4+1) {callback(constructError("invalid symbol information")); return;}

				var d =  new Date(m[1], m[2]-1, m[3], 12, 0);

				var MKT_STATUS = true;  // TODO!
				if (!MKT_STATUS) { obj.open = false; }

				if (d >= firstDate) { // make sure to include only quotes from days in the range.
					array[array.length] = { date: d, close: parseFloat(m[4]) };
				}
			}
			try {
				callback (obj);
			} catch (ex) {}
			
		}
		catch (ex)
		{
			callback ({error:true, errorString:"exception thrown when parsing stock chart data: " + ex});
		}
	}
	else
	{
		callback ({error:true, errorString:"XML request failed. no responseText"});
	}
}


function chartdata_xml_loaded (event, request, callback, symbol, code)
{
	if (request.responseXML) {
		try
		{
			var obj = {error:false, errorString:null, data:new Array};
	
			var INDEXSET = findChild(request.responseXML, "data-series");
			if (INDEXSET == null) {
				chartDataSourceState.errorDetected(symbol, code, "no <data-series>");
				fetchChartData(symbol, code, callback); // retry with alternative chart data source
				return;

				// callback(constructError("no <data-series>")); return;
			}
	
			var QUOTES = findChild (INDEXSET, "series");
			if (QUOTES == null) {
				chartDataSourceState.errorDetected(symbol, code, "no <series>");
				fetchChartData(symbol, code, callback); // retry with alternative chart data source
				return;
				
				// callback(constructError("no <series>")); return;
			}
			
			var array = obj.data;

			var nowDate = new Date();
			var firstDate = nowDate.addMonths(- XinMonths[code]);
			var latestDateInResult = firstDate;
	
			for (var child = QUOTES.firstChild; child != null; child = child.nextSibling)
			{
				if (child.nodeName == "p")
				{
					var DATETIME = child.getAttribute('ref');
					var CLOSE = findChild(child, "v");
	
					if (DATETIME == null || CLOSE == null)
					{
						chartDataSourceState.errorDetected(symbol, code, "Quote is missing a date or value");
						fetchChartData(symbol, code, callback); // retry with alternative chart data source
						return;
						
						// callback(constructError("Quote is missing a date or value"); return;
					}
					
					var d = createDateFromDATETIME(DATETIME);
					if (d >= firstDate) { // make sure to include only quotes from days in the range.
						array[array.length] = {date: d, close: parseFloat (CLOSE.firstChild.data)};
					}
					if (d >= latestDateInResult) {
						latestDateInResult = d;
					}
				}
			}

			if (nowDate.daysDifference(latestDateInResult) > chartDataSourceState.kDaysMissingThreshold) {
				chartDataSourceState.errorDetected(symbol, code, "Latest day in result is too old: " + latestDateInResult);
				fetchChartData(symbol, code, callback); // retry with alternative chart data source
				return;
			}

			try {
				callback (obj);
			} catch (ex) {
				// we do not want any eceptions to fire our exceptions from the callback
				// to fire our exception handler
			}
		}
		catch (ex)
		{
			var extxt = "exception thrown when parsing stock chart data: " + ex;
			chartDataSourceState.errorDetected(symbol, code, extxt);
			fetchChartData(symbol, code, callback); // retry with alternative chart data source

			// callback ({error:true, errorString:extxt});
		}
	}
	else
	{
		var extxt = "XML request failed. no responseXML";
		chartDataSourceState.errorDetected(symbol, code, extxt);
		fetchChartData(symbol, code, callback); // retry with alternative chart data source

		// callback ({error:true, errorString:extxt});
	}
}

function chartdata_xml_1d_1w_loaded (event, request, callback, symbol, code)
{
	if (request.responseXML) {
		try
		{
			var obj = {error:false, errorString:null, data:new Array,
						// Optionals:
						min_time:null, max_time:null, dayproportion:null,
						marketopen:null, marketclose:null};
	
			var INDEXSET = findChild(request.responseXML, "data-series");
			if (INDEXSET == null) {
				// chartDataSourceState.errorDetected(symbol, code, "no <data-series>");
				// fetchChartData(symbol, code, callback); // retry with alternative chart data source
				// return;

				callback(constructError("no <data-series>")); return;
			}

			var META = findChild (INDEXSET, "meta");
			if (META == null) {
				callback(constructError("no <meta>"));
				return;
			}
			var META_GMT = findChildWithAttrValue(META, "feature", "name", "gmtoffset");
			if (META_GMT == null) {
				callback(constructError("no <gmtoffset> tag found"));
				return;
			}
			var gmtoffset = META_GMT.firstChild.data;


			var META = findChild (INDEXSET, "reference-meta");
			if (META == null) {
				callback(constructError("no <reference-meta>"));
				return;
			}
			obj.min_time = createDateFromTIMESTAMP(gmtoffset, findChild(META, "min").firstChild.data);
			obj.max_time = createDateFromTIMESTAMP(gmtoffset, findChild(META, "max").firstChild.data);

			var RANGES = findChild(META, "ranges");
			if (RANGES != null) {	// for 5d, but not for 1d !!
				var RANGE = RANGES.lastChild;
				obj.marketopen = createDateFromTIMESTAMP(gmtoffset, RANGE.getAttribute('min'));
				obj.marketclose = createDateFromTIMESTAMP(gmtoffset, RANGE.getAttribute('max'));
			}
			else {
				obj.marketopen = obj.min_time;
				obj.marketclose = obj.max_time;
			}


			var QUOTES = findChild (INDEXSET, "series");
			if (QUOTES == null) {
				// chartDataSourceState.errorDetected(symbol, code, "no <series>");
				// fetchChartData(symbol, code, callback); // retry with alternative chart data source
				// return;
				
				callback(constructError("no <series>")); return;
			}
			
			var array = obj.data;

			for (var child = QUOTES.firstChild; child != null; child = child.nextSibling)
			{
				if (child.nodeName == "p")
				{
					var TIMESTAMP = child.getAttribute('ref');
					var CLOSE = findChild(child, "v");
	
					if (TIMESTAMP == null || CLOSE == null)
					{
						// chartDataSourceState.errorDetected(symbol, code, "Quote is missing a date or value");
						// fetchChartData(symbol, code, callback); // retry with alternative chart data source
						// return;

						callback(constructError("Quote is missing a date or value")); return;
					}
					
					var d = createDateFromTIMESTAMP(gmtoffset, TIMESTAMP);
					array[array.length] = {date: d, close: parseFloat (CLOSE.firstChild.data)};
				}
			}

			try {
				callback (obj);
			} catch (ex) {
				// we do not want any eceptions to fire our exceptions from the callback
				// to fire our exception handler
			}
		}
		catch (ex)
		{
			var extxt = "exception thrown when parsing stock chart data: " + ex;
			// chartDataSourceState.errorDetected(symbol, code, extxt);
			// fetchChartData(symbol, code, callback); // retry with alternative chart data source

			callback ({error:true, errorString:extxt});
		}
	}
	else
	{
		var extxt = "XML request failed. no responseXML";
		// chartDataSourceState.errorDetected(symbol, code, extxt);
		// fetchChartData(symbol, code, callback); // retry with alternative chart data source

		callback ({error:true, errorString:extxt});
	}
}

function createDateFromDATETIME (string)
{
	// skip time of day
	var year = parseInt(string.substr(0,4), 10);
	var month = parseInt(string.substr(4,2), 10) - 1;
	var day = parseInt(string.substr(6,2), 10);
	
	var date =  new Date(year, month, day, 12, 0);
	
	return date;
}

function createDateFromTIMESTAMP (gmtoffsetInSec, timestamp)
{
	var d = new Date();
	d.setTime((Number(timestamp) + Number(gmtoffsetInSec) + gTimezoneOffsetInMin*60) * 1000);
	return d;
}

/*
    http://www.json.org/json.js
    2006-04-28

    This file adds these methods to JavaScript:

        object.toJSONString()

            This method produces a JSON text from an object. The
            object must not contain any cyclical references.

        array.toJSONString()

            This method produces a JSON text from an array. The
            array must not contain any cyclical references.

        string.parseJSON()

            This method parses a JSON text to produce an object or
            array. It will return false if there is an error.
*/
(function () {
    var m = {
            '\b': '\\b',
            '\t': '\\t',
            '\n': '\\n',
            '\f': '\\f',
            '\r': '\\r',
            '"' : '\\"',
            '\\': '\\\\'
        },
        s = {
            array: function (x) {
                var a = ['['], b, f, i, l = x.length, v;
                for (i = 0; i < l; i += 1) {
                    v = x[i];
                    f = s[typeof v];
                    if (f) {
                        v = f(v);
                        if (typeof v == 'string') {
                            if (b) {
                                a[a.length] = ',';
                            }
                            a[a.length] = v;
                            b = true;
                        }
                    }
                }
                a[a.length] = ']';
                return a.join('');
            },
            'boolean': function (x) {
                return String(x);
            },
            'null': function (x) {
                return "null";
            },
            number: function (x) {
                return isFinite(x) ? String(x) : 'null';
            },
            object: function (x) {
                if (x) {
                    if (x instanceof Array) {
                        return s.array(x);
                    }
                    var a = ['{'], b, f, i, v;
                    for (i in x) {
                        v = x[i];
                        f = s[typeof v];
                        if (f) {
                            v = f(v);
                            if (typeof v == 'string') {
                                if (b) {
                                    a[a.length] = ',';
                                }
                                a.push(s.string(i), ':', v);
                                b = true;
                            }
                        }
                    }
                    a[a.length] = '}';
                    return a.join('');
                }
                return 'null';
            },
            string: function (x) {
                if (/["\\\x00-\x1f]/.test(x)) {
                    x = x.replace(/([\x00-\x1f\\"])/g, function(a, b) {
                        var c = m[b];
                        if (c) {
                            return c;
                        }
                        c = b.charCodeAt();
                        return '\\u00' +
                            Math.floor(c / 16).toString(16) +
                            (c % 16).toString(16);
                    });
                }
                return '"' + x + '"';
            }
        };

    Object.prototype.toJSONString = function () {
        return s.object(this);
    };

    Array.prototype.toJSONString = function () {
        return s.array(this);
    };
})();

String.prototype.parseJSON = function () {
    try {
        return !(/[^,:{}\[\]0-9.\-+Eaeflnr-u \n\r\t]/.test(
                this.replace(/"(\\.|[^"\\])*"/g, ''))) &&
            eval('(' + this + ')');
    } catch (e) {
        return false;
    }
};



//
// Fault-tolerance Feature for loading chart data from one of two sources.
//
// Class for holding state when and for how long to use alternative chart data source
//
//
function ChartDataSourceState()
{
	this.symbol = null;
	this.code = null;	// 1m, ...
	this.counter = 0;

	this.kCountdown = 10;
	this.kDaysMissingThreshold = 5;
}

ChartDataSourceState.prototype.errorDetected = function (symbol, code, message)
{
	this.symbol = symbol;
	this.code = code;
	this.counter = this.kCountdown;
	alert(message + " -- symbol = " + symbol + ", code = " + code);
}

ChartDataSourceState.prototype.countdown = function (symbol, code)
{
	if (this.symbol == symbol && this.code == code) {
		if (this.counter > 0) {
			this.counter--;
			alert("Using alternative chart data source. Counter = " + this.counter + " -- symbol = " + symbol + ", code = " + code);
			return true;
		}
		return false;
	}
	// if symbol or code does not match then countdown is obsolete.
	else {
		this.counter = 0;
		return false;
	}
}

var chartDataSourceState = new ChartDataSourceState();
